var gulp         = require('gulp'),
    themes       = require('fs').readdirSync('themes'),
    isProduction = false,
    $            = require('gulp-load-plugins')(),
    rimraf       = require('rimraf'),
    cleanCSS     = require('gulp-clean-css');
    
var globs = {
    less: 'themes/**/css/*.less',
    css: 'themes/**/*.css',
    js: 'themes/**/*.{js,map}',
    images: 'themes/**/*.{png,gif,jpg,jpeg,svg}',
    misc: ['themes/**/*.{ico,eot,woff,ttf,php,txt}', 'themes/**/.htaccess']
};

var target = '../panel/themes';

gulp.task('css', async function() {
    gulp.src(globs.less)
        .pipe($.filter(['**/*.less', '!**/_*.less']))
        .pipe($.less())
        .pipe($.autoprefixer('last 3 versions'))
        .pipe($.if(isProduction, cleanCSS()))
        .pipe(gulp.dest(target));
    gulp.src(globs.css)
        .pipe($.if(isProduction, cleanCSS()))
        .pipe(gulp.dest(target));
});
gulp.task('js', async function () {
    return gulp.src(globs.js)
        .pipe($.if(isProduction, $.uglify()))
        .pipe(gulp.dest(target));
});
gulp.task('images', async function() {
    return gulp.src(globs.images)
        .pipe(gulp.dest(target));
});
gulp.task('misc', async function () {
    return gulp.src(globs.misc).pipe(gulp.dest(target));
});

gulp.task('clean', async function () {
    rimraf.sync('../panel/assets/*');
    for (var i = 0, l = themes.length; i < l; i++) {
        rimraf.sync(target + '/' + themes[i]);
    }
});

gulp.task('setProduction', async function() {
    isProduction = true;
});

gulp.task('default', gulp.series('clean', 'js', 'css', 'images', 'misc'));
gulp.task('dist', gulp.series('setProduction', 'default'));
